package TextDatabase;

/**
 * @author Costas Vandikas (vandikas@csd.uoc.gr)
 */
 
import java.util.Vector;
import java.util.Iterator;
import java.io.FileReader;
import java.io.BufferedReader;
import SearchOperations.Query;

/**
 * The Document is a tree set of keywords
 */
public class Document extends KeywordTreeSet implements Comparable
{
	/**
     * Holds the filename of the document
     */
	private String _fileName;
	
	/**
     * Holds the document TreeSet that contains this document
     */
    private DocumentTreeSet _documentTreeSet;

	/**
     * Initializes the Document from a file
     */
	public Document( DocumentTreeSet DocumentTreeSet, String FileName )
	{
		_documentTreeSet = DocumentTreeSet;
		setFileName( FileName );
	}

	/**
     * Gets the Document filename
     */
	public String getFileName()
	{
		return _fileName;
	}

	/**
     * Sets the document's filename and keyword tree set
     */
	public void setFileName( String FileName )
	{
		_fileName = FileName;
		PopulateKeywords();
	}

	/**
     * Sets the document's keyword tree set while eliminating the stop words
     */
	private void PopulateKeywords()
	{
		StopWordTreeSet stopWordTreeSet = new StopWordTreeSet();

		try
		{
			KeywordTreeSet globalKeywordTreeSet = 
				_documentTreeSet.getGlobalKeywordTreeSet();
				
			WordReader wordReader = new WordReader(	
				DocumentTreeSet.documentDirectory + "\\" + _fileName);
					
			while( wordReader.ready() )
			{
				String keywordName = wordReader.ReadWord();
				if ( keywordName != "" && 
					!stopWordTreeSet.contains(keywordName) )
				{
					if ( addKeyword( keywordName ) == 1)
						globalKeywordTreeSet.addKeyword( keywordName );
				}
			}
			
			wordReader.close();
			_documentTreeSet.setGlobalKeywordTreeSet( globalKeywordTreeSet );
		}
		catch( Exception x )
		{
			System.out.println (x.toString ());
		}

		stopWordTreeSet = null;	
	}

	/**
     * Compares the Document to another document
     */
	public int compareTo( Object Obj )
	{
		return ( _fileName.compareTo( ((Document)Obj)._fileName ) );
	}

	/**
     * Checks if the Document is equal to another Document
     */
	public boolean equals( Object Obj )
	{
		return ( _fileName == ((Document)Obj)._fileName );
	}

	/**
     * Checks if a document is relevant to a query by evaluating the
     * keywords and the operands of the query
     */
	public boolean isRelevant( Query Query )
	{
		Iterator iterator = Query.iterator();
		boolean[] booleanWeights = new boolean[Query.size()];
		for ( int i=0; iterator.hasNext(); i++ )
			booleanWeights[i] = contains( 
				((Keyword)iterator.next()).toString() );
		
		return Query.evaluate(booleanWeights);
	}

	/**
     * Returns the documents filename
     */
	public String toString()
	{
		return _fileName;
	}

	/**
     * Returns the document's filename contents
     */
	public String getText()
	{
		String text = "";
		try
		{
			BufferedReader reader = new BufferedReader( new FileReader( 
				DocumentTreeSet.documentDirectory + "/" + _fileName ) );
			while ( reader.ready() )
				text += reader.readLine() + "\n";
		}
		catch( Exception x )
		{
			System.out.println (x.toString());
		}

		return text;
	}
	
	/**
     * Vectorizes the contents of the tree set
     */
	public Vector toVector()
	{
		Vector mainVector = new Vector();
		Iterator iterator = iterator();
		while( iterator.hasNext() )
		{
			Vector innerVector = new Vector();
			Keyword keyword = ((Keyword)iterator.next());
			innerVector.add( keyword.toString() );
			innerVector.add( String.valueOf(keyword.getFrequency()) );
			innerVector.add( String.valueOf(getNormalizedFrequency(keyword)));
			double inverseFrequency = 
				_documentTreeSet.getInverseDocumentFrequency(keyword);
			innerVector.add( String.valueOf(inverseFrequency) );
			innerVector.add( String.valueOf(
					getTermWeight(keyword,inverseFrequency)) );
			mainVector.add( innerVector );
		}

		return mainVector;
	}
	
	/**
     * Get's the weight of a term for the specified document
     */
	public double getTermWeight( Keyword Keyword, double InverseFrequency )
	{
		return getTermWeight( Keyword.getKeywordName(), InverseFrequency );
	}

	/**
     * Gets the weight of term for the specifeid document.
     * DocumentTermWeight = 
     *	term's normalized frequency(document) 
     *		* term's inverse document frequency
     */
	public double getTermWeight( String KeywordName, double InverseFrequency )
	{
		return getNormalizedFrequency(KeywordName) * InverseFrequency;
	}
}