package TextDatabase;

import java.util.TreeSet;
import java.util.Iterator;
import java.io.File;
import java.io.FileFilter;
import SearchOperations.Query;
import UserInterface.Application;

/**
 * @author Costas Vandikas (vandikas@csd.uoc.gr)
 */

/**
 * A DocumentTreeSet is a TreeSet of Documents
 */
public class DocumentTreeSet extends TreeSet
{
	/**
     * Holds the directory of the documents
     */
	public static String documentDirectory = "documents";
	
	/**
	 * Holds the global keyword tree set
	 */
	private KeywordTreeSet _globalKeywordTreeSet;

	/**
     * Initializes the Document Tree Set by reading all the documents
     * held in the document directory
     */
	public DocumentTreeSet()
	{
		System.out.println( "Please wait while the caching of documents is completed..." );
		_globalKeywordTreeSet = new KeywordTreeSet();
		File[] documentFiles = getDocumentFiles();
		for ( int i=0; i<documentFiles.length; i++ )
			add( new Document( this, documentFiles[i].getName() ) );
		
		System.out.println( "Caching of documents completed." );
	}
	
	public KeywordTreeSet getGlobalKeywordTreeSet()
	{
		return _globalKeywordTreeSet;
	}
	
	public void setGlobalKeywordTreeSet( KeywordTreeSet GlobalKeywordTreeSet )
	{
		_globalKeywordTreeSet = GlobalKeywordTreeSet;
	}

	/**
     * Finds all the documents held in the document directory
     */
	private File[] getDocumentFiles()
	{
		File file = new File( documentDirectory );
		
		if ( file.isDirectory () )
			return file.listFiles ( new DocumentFileFilter() );

		return null;
	}

	/**
     * Returns the document with the corresponding document name
     */
	public Document getDocument( String DocumentName )
	{
		Iterator iterator = iterator();
		while ( iterator.hasNext() )
		{
			Document document = (Document)iterator.next();
			if ( document.toString().equals(DocumentName) )
				return document;
		}
		
		return null;
	}

	/**
     * Implements a class of File Filter that excludes all files which
     * are not documents (*.txt)
     */
	private class DocumentFileFilter implements FileFilter
	{
		public boolean accept(File File)
		{
			if (File.isDirectory())
				return false;
			else
			{
				int index = File.getName().lastIndexOf(".");
				String extension = ( index != -1 ) ? 
				File.getName().substring( index ) : File.getName();
				if (extension != null)
					return ((extension.equals(".txt")));
			}

			return false;
		}
	}

	/**
     * Returns the number of times a keyword exists in the Global Tree Set
     */
	private int getKeywordCount( String KeywordName )
	{
		Keyword keyword = _globalKeywordTreeSet.getKeyword( KeywordName );
		if ( keyword != null )
			return keyword.getFrequency();
		
		return 0;
	}

	/**
     * Returns the inverse document frequency 
     * Inverse Document Frequency = log2( Number Of Documents divided by
     * the number of times a keyword exists in the Tree Set
     */
	public double getInverseDocumentFrequency( Keyword Keyword )
	{
		return getInverseDocumentFrequency( Keyword.getKeywordName() );
	}

	/**
     * Returns the inverse document frequency 
     * Inverse Document Frequency = log2( Number Of Documents divided by
     * the number of times a keyword exists in the Tree Set
     */
	public double getInverseDocumentFrequency( String KeywordName )
	{
		return getInverseDocumentFrequency(getKeywordCount(KeywordName));
	}
	
	public double getInverseDocumentFrequency( int KeywordCount )
	{
		return log( (double)size()/KeywordCount, 10 );
	}

	/**
     * Implements the log function by changing the base of the natural
     * logarithm
     */
	private double log( double number, int base )
	{
		return Math.log(number)/Math.log(base);
	}
	
	public double getDocumentRank( Query Query, Document Document )
	{
		if ( Application.Mode == 0 || Application.Mode == 1 )
			return getFullDocumentRank( Query, Document );
			
		if ( Application.Mode == 2 )
			return getAlternateDocumentRank( Query, Document );
			
		return 0;
	}
	
	/**
     * Gets the document rank for the specified query.
     *
     *                sum( documentTermWeight * queryTermWeight )
     * Rank = --------------------------------------------------------------
     *	      sqrt(sum(documentTermWeight^2)) * sqrt(sum(queryTermWeight^2))
     */
	public double getAlternateDocumentRank( Query Query, Document Document )
	{
		double numerator = 0;
		double denumerator1 = 0;
		double denumerator2 = 0;
		Iterator iterator = Query.iterator();

		while ( iterator.hasNext() )
		{
			Keyword keyword = (Keyword)iterator.next();
			double inverseFrequency = 
				getInverseDocumentFrequency(keyword.toString());
			
			double documentTermWeight = 
				Document.getTermWeight( keyword, inverseFrequency );
			
			double queryTermWeight = 
				Query.getTermWeight( keyword.toString(), inverseFrequency );
			
			numerator += documentTermWeight * queryTermWeight;
			denumerator1 += documentTermWeight * documentTermWeight;
			denumerator2 += queryTermWeight * queryTermWeight;
		}

		return ( numerator /
			(Math.sqrt(denumerator1)*Math.sqrt(denumerator2)))* 100;
	}
	
	public double getFullDocumentRank( Query Query, Document Document )
	{
		double numerator = 0;
		double denumerator1 = 0;
		double denumerator2 = 0;
		Iterator termIterator = _globalKeywordTreeSet.iterator();

		while ( termIterator.hasNext() )
		{
			Keyword keyword = (Keyword)termIterator.next();
				
			double inverseFrequency = 
				getInverseDocumentFrequency( keyword.getFrequency() );
				
			double documentTermWeight = 
				Document.getTermWeight( keyword.toString(), inverseFrequency );
	
			double queryTermWeight = 
				Query.getTermWeight( keyword.toString(), inverseFrequency );
			
			numerator += documentTermWeight * queryTermWeight;
			denumerator1 += documentTermWeight * documentTermWeight;
			denumerator2 += queryTermWeight * queryTermWeight;
		}
		
		return (( numerator /
			(Math.sqrt(denumerator1)*Math.sqrt(denumerator2)))* 100);
	}
}