package SearchOperations;

/**
 * @author Costas Vandikas (vandikas@csd.uoc.gr)
 */

import java.util.Iterator;
import SearchOperations.Query;
import TextDatabase.WordReader;
import TextDatabase.Keyword;
import TextDatabase.KeywordTreeSet;
import UserInterface.Application;

/**
 * Implementation of the user's query
 */
public class Query extends KeywordTreeSet
{
	/**
     * Holds the operators the query can identify
     */
	public static final char[] activeOperators = { '+', '|' };

	/**
     * Holds the raw query expression
     */
	private String _expression;

	/**
     * Initializes the query from a raw text expression
     */
	public Query( String Text )
	{
		setExpression( Text );
	}

	/**
     * Set's the query expression
     */
	public void setExpression( String Text )
	{
		_expression = Text;
		setKeywords();
	}

	/**
     * Identifies the keywords in a query and stores them.
     * A keyword array is supplied by splitting the expression in every
     * space character.
     */
	private void setKeywords()
	{
		_expression = preprocess( _expression );
		String[] _keywords = _expression.trim().split( " " );
		for ( int i=0; i<_keywords.length; i++ )
		{
			_keywords[i] = _keywords[i].replace( 
				activeOperators[0], ' ' ).replace( 
					activeOperators[1], ' ' ).trim();
			addKeyword( _keywords[i] );
		}
	}
	
	private String preprocess( String inputString )
	{
		inputString = inputString.replace( ',', ' ' ).replace( '-', ' ' ).replace( '.', ' ' );
		String result = "";
		
		for ( int i=0; i < inputString.length(); i++ )
		{
			int ch = inputString.charAt(i);
			
			if ( ch >= WordReader.CapitalA && ch <= WordReader.CapitalZ )
				ch += WordReader.CapitalToSmallDistance;
			
			if ( ch >= WordReader.SmallA && ch <= WordReader.SmallZ )
				result += (char)ch;
				
			if ( ch == activeOperators[0] || ch == activeOperators[1] )
				result += (char)ch;
			
			if ( ch == ' ' )
				result += (char)ch;
		}

		return result;
	}

	/**
     * Gets the raw expression
     */
	public String getExpression()
	{
		return _expression.trim();
	}

	/**
     * Evaluates the query by using the boolean weights provided by the document
     * 1. For every document we get a boolean weight claiming wheter or not it 
     * contains a keyword.
     * 2. In the raw expression we replace each keyword for the proper weight
     * 3. The expression is evaluated by identifying each operator and replacing
     * it and it's operands with the result of the evaluation. First we 
     * calculate the and operators and than the or operators.
     * 4. In the end the length of the expression should be one. That one
     * character holds the result of the expression.
     */
	public boolean evaluate( boolean[] BooleanWeights )
	{
		StringBuffer evaluation = fullTrim(assignValues(BooleanWeights));

		for ( int i=0, j=0; 
			evaluation.indexOf("+") != -1 || evaluation.length() != 1; )
		{
			if ( evaluation.charAt(i) == activeOperators[j] )
				evaluation = evaluation.replace( i-1, i+2, 
					calculate( evaluation.substring(i-1, i+2) ) );
			else
				i++;

			if ( evaluation.indexOf("+") == -1 && j == 0 )
			{
				i=0; j++;
			}
		}

		return ( evaluation.charAt(0) == '1' ) ? true : false;
	}

	/**
     * Assings the boolean weights on the raw expression
     */
	private String assignValues( boolean[] BooleanWeights )
	{
		String expression = _expression.trim();
		Iterator iterator = iterator();
		for ( int i=0; iterator.hasNext(); i++ )
			expression = replaceFirst( expression, 
				((Keyword)iterator.next()).toString(), 
					(BooleanWeights[i]) ? "1" : "0" );
		
		return expression;
	}
	
	private String replaceFirst( String InputString, String OldValue, 
		String NewValue )
	{
		StringBuffer stringBuffer = new StringBuffer(InputString);
		int index = stringBuffer.indexOf( OldValue );
		int lastIndex = ( stringBuffer.indexOf( " ", index ) == -1 ) 
				? stringBuffer.length() : stringBuffer.indexOf( " ", index );
				
		if ( index == 0 && match( index, lastIndex, 
			stringBuffer.toString(), OldValue ) )
			stringBuffer.replace( index, index + OldValue.length(), 
				NewValue ).toString();
		
		index = (index - 1 > 0 ) ? index-1 : 0;
				
		for ( int i=index; i<stringBuffer.length(); i++ )
		{
			if ( stringBuffer.charAt(i) == activeOperators[0] ||
					stringBuffer.charAt(i) == activeOperators[1] )
			{
				index = stringBuffer.indexOf(OldValue);
				lastIndex = ( stringBuffer.indexOf( " ", i ) == -1 ) 
					? stringBuffer.length() : stringBuffer.indexOf( " ", i );

				if ( index != -1 && index == i+1 
					&& match( index, lastIndex, 
						stringBuffer.toString(), OldValue) )
					stringBuffer.replace( index, index + OldValue.length(), 
							NewValue ).toString();
			}
		}
	
		return stringBuffer.toString();
	}
	
	private boolean match( int Index, int LastIndex, String InputString, 
		String Pattern)
	{
		int j=0;
		for ( int i=Index; i<LastIndex; i++, j++ )
		{
			if ( InputString.charAt(i) != Pattern.charAt(j) )
				return false;
		}
		
		return ( j == Pattern.length() );
	}

	/**
     * Trims all spaces from an expression
     */
	private StringBuffer fullTrim( String String )
	{
		StringBuffer stringBuffer = new StringBuffer( String );
		for ( int i=stringBuffer.length()-1; i>=0; i-- )
			if ( stringBuffer.charAt(i) == ' ' )
				stringBuffer = stringBuffer.deleteCharAt(i);
		return stringBuffer;
	}

	/**
     * Calculates a boolean string with the specified operator.
     * Example calculate( "0+1" ) = 0
     */
	private String calculate( String String )
	{
		char operator = String.substring(1,2).charAt(0);
		int	operand1 = Integer.parseInt(String.substring(0,1));
		int	operand2 = Integer.parseInt(String.substring(2,3));

		switch( operator )
		{
			case '+' :
				return String.valueOf( operand1 & operand2 );
			case '|' :
				return String.valueOf( operand1 | operand2 );
		}

		return "";
	}
	
	/**
     * Gets the weight of term for the specifed query
     * QueryTermWeight = 
     *	(0.5 + (0.5*term's normalized frequency(query))) * 
     *	term's inverse document frequency
     */
	public double getTermWeight( String KeywordName, double inverseFrequency )
	{
		if ( Application.Mode == 0 || Application.Mode == 2 )
			return (0.5 + 0.5*getNormalizedFrequency(KeywordName))*
				inverseFrequency;
				
		if ( Application.Mode == 1 )
			return (getNormalizedFrequency(KeywordName))*
				inverseFrequency;
		
		return 0;
	}
}